pub mod api;
pub mod display;
pub mod dither;
pub mod errors;

use std::path::PathBuf;

use crate::display::{EInkPanel, FakeEInk, Wrapper};
use crate::dither::{DitherMethod, DitheredImage, Palette};
use clap::{Args, Parser, Subcommand};
use image::RgbImage;
use tracing::{error, info};

/// Display images on E-Ink Displays
#[derive(Debug, Parser)]
#[command(version, about)]
struct Cli {
    #[command(subcommand)]
    command: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
    /// Convert an image into a dithered form.
    Convert(ConvertArgs),
    /// Load a single image
    Show,
    /// Display a test pattern
    Test,
    /// Start the HTTP server
    Serve,
}

#[derive(Args, Debug)]
struct ConvertArgs {
    #[arg(short, long, default_value_t = DitherMethod::NearestNeighbor)]
    dither_method: DitherMethod,
    input_file: PathBuf,
    output_file: PathBuf,
}

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    tracing_subscriber::fmt::init();
    let cli = Cli::parse();
    println!("CLI {cli:?}");

    match cli.command {
        Command::Convert(a) => {
            let input = image::ImageReader::open(a.input_file)?
                .decode()?
                .into_rgb8();
            let mut method = a.dither_method.get_ditherer();

            let mut result = DitheredImage::new(
                input.width(),
                input.height(),
                Palette::Default.value().to_vec(),
            );
            method.dither(&input, &mut result);

            result.into_rgbimage().save(a.output_file)?;
        }
        Command::Show => {
            let img: RgbImage = image::ImageReader::open("image.png")?.decode()?.into();
            error!("HI");
            let mut display = FakeEInk {};

            let mut eink_buf = DitheredImage::default();
            let mut dither = DitherMethod::Atkinson.get_ditherer();

            dither.dither(&img, &mut eink_buf);
            display.display(&eink_buf)?;
        }
        Command::Test => {
            let mut display = Wrapper::new()?;

            display.test()?;
        }
        Command::Serve => {
            let display = FakeEInk {};

            let ctx = api::Context::new(Box::new(display));

            let app = api::router().with_state(ctx);
            let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await?;
            info!("Listening on 0.0.0.0:3000");
            axum::serve(listener, app).await?;
        }
    }

    Ok(())
}
